# Contributing to landlab

Thank you for contributing to the landlab! We appreciate
your help as this is largely as volunteer effort! :heart: :heart: :heart:

# How to contribute

## Reporting Bugs

Before creating a bug report, please do at least a cursory check that the
bug has not already been reported. If it has, add a comment to the existing
issue instead of opening a new one.

### Submitting a Bug Report

Bugs are tracked as
[GitHub issues](https://guides.github.com/features/issues/). After you've
determined you've found a new bug, please open a
[new issue](https://github.com/landlab/landlab/issues).

Explain the problem and include additional details to help maintainers
reproduce the problem. Here are some items that will make it easier
to track down the source of the problem.

*  **Use a clear and descriptive title** for the issue that identifies the
   problem.
*  **Describe the exact steps that reproduce the problem**.
*  **Provide a [minimal example](https://stackoverflow.com/help/mcve)
   that demonstrates the steps** as, for example, a bash script
   along with input files. This example should reproduce your
   problem with as few lines of code as possible and easily
   reproducible my another person.
*  **Describe the behavior you are seeing after these steps**.
*  **Describe the behavior you expect to see after these steps**.

Additionally, the answers to the following questions about your run
environment will be helpful.

*  **Which version of landlab are you using?** This could be a specific
   git sha or a release number.
*  **What is he name and version of you OS?**
*  **What compiler are you using?**
*  **How did you build landlab (if using the development version)?**


## Submitting Changes

:tada: Whoa! This is great! We love it when folks contibute code! :tada:

Changes to landlab should be submitted as 
[pull requests](http://help.github.com/pull-requests/)).

*  Create a GitHub issue that describes what you propose to do.
*  Create a topic branch that contains your changes.
*  Open a new [GitHub pull request](https://github.com/landlab/landlab/pull/new/master).
*  Ensure the pull request description clearly describes the problem
   and solution.  Include the relevant issue number.

## Styleguides

Use the [PEP8 style guide](https://www.python.org/dev/peps/pep-0008/).
You may want to use tools like
[Prospector](http://prospector.landscape.io/en/master/) to help out
with this.

### Git Commit Messages

* Use the present tense ("Add feature" not "Added feature")
* Use the imperative mood ("Move cursor to..." not "Moves cursor to...")
* Limit the first line to 72 characters or less
* Reference issues and pull requests liberally
* For fun, consider starting the commit message with an applicable emoji:
    * :art: `:art:` when improving the format/structure of the code
    * :racehorse: `:racehorse:` when improving performance
    * :non-potable_water: `:non-potable_water:` when plugging memory leaks
    * :memo: `:memo:` when writing docs
    * :penguin: `:penguin:` when fixing something on Linux
    * :apple: `:apple:` when fixing something on macOS
    * :checkered_flag: `:checkered_flag:` when fixing something on Windows
    * :bug: `:bug:` when fixing a bug
    * :fire: `:fire:` when removing code or files
    * :green_heart: `:green_heart:` when fixing the CI build
    * :white_check_mark: `:white_check_mark:` when adding tests
    * :shirt: `:shirt:` when removing linter warnings

### Pull Request Messages

  * Rename the pull request and provide a comment that synthesizes what 
    the pull request changes or adds. This helps us synthesize what 
    changes have occured between Landlab releases. 

## Adding new components

If you would like to create a new component, we have just a few
conventions that we would like you to follow.

* Create a new folder under `landlab/components` that will hold your
  new component.
* Be aware that your component will have to be able to work with both
  Python 2 and Python 3.

### Python 2 vs. Python 3

* We use `six` to maintain a single code base that will work with
  both Python 2 and 3.
* All tests will be run with Python 2.7, 3.5, and 3.6.
* If you need to introduce a new dependency, that dependency must
  be compatible with Python 2 and 3.

### Documentation

* All public functions, classes, methods, etc. must have a docstring
  that follows the [numpydoc](https://github.com/numpydoc/numpydoc)
  conventions.
* Every `.py` file must contain a module-level docstring at the top
  of the file that describes what the purpose of the file is. 
* Additionally, you will need to add a ReStructuredText (.rst) file
  so that the documentation of your new part of Landlab can be 
  autogenerated. See [this section of the creating a component](https://github.com/landlab/landlab/wiki/Develop-your-own-component#getting-your-component-into-the-documentation) documentation 
  for an example of how to do this. 

### Testing

* All contributed code must be well tested. This should be done 
  through both doctests as well as more standard unit tests through 
  [`pytest`](http://docs.pytest.org).
* Doctests should be short, easy-to-read tests that are instructive
  to a user.
* Unit tests should be significanly more extensive and give your
  new code thorough testing. Ideally your tests will test what 
  happens within every `if`, `elif`, or `else `, and every `try` or
  `except` block. Additionally, unless there is a specific reason your
  component or utility can only work with one landlab grid type, the 
  tests should verify that it can work with multiple model grid types 
  (e.g. both `HexModelGrid` and `RasterModelGrid`).
* Your unit tests should verify that the component or utility you are 
  creating does exactly what it is expected to do. This means you will 
  probably want to create a very small (e.g. 5x5 model grid and hand
  calculate what the correct answer is). Then assert that your code
  reproduces that answer. 
* Unit tests [must be discoverable by `pytest`](http://docs.pytest.org).
  This means that the unit tests should be in folders called `test` 
  within the component or utility folder, in `.py` files that start with 
  the name `test` and within functions with names that begin with the 
  word `test`. 
* For more information on constructing unit tests, see [this section](https://github.com/landlab/landlab/wiki/Develop-your-own-component#writing-docstring-and-unit-tests-for-your-component-or-utility)
  of the User Guide. 

Thanks! :heart: :heart: :heart:

The landlab team
